@@ -4,7 +4,7 @@
 # performance processing of geographical data that uses GRASS GIS for
 # computational tasks. For details, see https://actinia.mundialis.de/
 #
-# Copyright (c) 2016-2021 Sören Gebbert and mundialis GmbH & Co. KG
+# Copyright (c) 2016-2024 Sören Gebbert and mundialis GmbH & Co. KG
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -56,11 +56,18 @@ __author__ = (
     " Anika Weinmann"
 )
 __copyright__ = (
-    "Copyright 2016-2022, Sören Gebbert and mundialis GmbH & Co. KG"
+    "Copyright 2016-2024, Sören Gebbert and mundialis GmbH & Co. KG"
 )
 __maintainer__ = "mundialis"
 
 
+def get_param_stdin_part(text):
+    """Function to get method and filter from parameter value"""
+    for delimiter in ["::", " ", "+", "-", "*", ":", "(", ")"]:
+        text = text.split(delimiter, 1)[0]
+    return text
+
+
 class ProcessChainConverter(object):
     """
     Convert the process chain description into a process list that can be
@@ -143,9 +150,9 @@ class ProcessChainConverter(object):
         self.webhook_finished = None
         self.webhook_update = None
         self.webhook_auth = None
+        self.stdin_num = 0
 
     def process_chain_to_process_list(self, process_chain):
-
         if not process_chain:
             raise AsyncProcessError("Process chain is empty")
 
@@ -188,7 +195,6 @@ class ProcessChainConverter(object):
 
         # Check for the webhooks
         if "webhooks" in process_chain:
-
             if "finished" in process_chain["webhooks"]:
                 self.webhook_finished = process_chain["webhooks"]["finished"]
                 self._check_if_webhook_exists(
@@ -206,7 +212,6 @@ class ProcessChainConverter(object):
                 )
 
         for process_descr in process_chain["list"]:
-
             if "module" in process_descr:
                 module = self._create_module_process(process_descr)
                 if module:
@@ -444,20 +449,28 @@ class ProcessChainConverter(object):
         """
         rvf_downimport_commands = list()
         url = entry["import_descr"]["source"]
-        gdis = GeoDataDownloadImportSupport(
-            config=self.config,
-            temp_file_path=self.temp_file_path,
-            download_cache=self.temp_file_path,
-            message_logger=self.message_logger,
-            send_resource_update=self.send_resource_update,
-            url_list=[
-                url,
-            ],
-        )
-        download_commands, import_file_info = gdis.get_download_process_list()
-        rvf_downimport_commands.extend(download_commands)
+
+        if not url.startswith("/vsicurl/"):
+            gdis = GeoDataDownloadImportSupport(
+                config=self.config,
+                temp_file_path=self.temp_file_path,
+                download_cache=self.temp_file_path,
+                message_logger=self.message_logger,
+                send_resource_update=self.send_resource_update,
+                url_list=[
+                    url,
+                ],
+            )
+            (
+                download_commands,
+                import_file_info,
+            ) = gdis.get_download_process_list()
+            rvf_downimport_commands.extend(download_commands)
+            input_source = import_file_info[0][2]
+        else:
+            input_source = url
+
         map_name = entry["value"]
-        input_source = import_file_info[0][2]
         layer = None
 
         if "vector_layer" in entry["import_descr"]:
@@ -609,6 +622,7 @@ class ProcessChainConverter(object):
         if self.message_logger:
             self.message_logger.info(str(module_descr))
         params = []
+        param_stdin_funcs = {}
 
         if "id" not in module_descr:
             raise AsyncProcessError(
@@ -635,12 +649,12 @@ class ProcessChainConverter(object):
 
         if "inputs" in module_descr:
             self._add_grass_module_input_parameter_to_list(
-                module_descr, params, id
+                module_descr, params, param_stdin_funcs, id
             )
 
         if "outputs" in module_descr:
             self._add_grass_module_output_parameter_to_list(
-                module_descr, params, id, module_name
+                module_descr, params, param_stdin_funcs, id, module_name
             )
 
         if "flags" in module_descr:
@@ -683,6 +697,7 @@ class ProcessChainConverter(object):
                 executable=module_name,
                 executable_params=params,
                 stdin_source=stdin_func,
+                param_stdin_sources=param_stdin_funcs,
                 id=id,
             )
 
@@ -709,16 +724,63 @@ class ProcessChainConverter(object):
                 "The stdin option in id %s misses the ::" % str(id)
             )
         object_id, method = module_descr["stdin"].split("::")
-        if "stdout" == method is True:
-            stdin_func = self.process_dict[object_id].stdout
-        elif "stderr" == method is True:
-            stdin_func = self.process_dict[object_id].stderr
+        if "stdout" == method:
+            stdin_func = self.process_dict[object_id].get_stdout
+        elif "stderr" == method:
+            stdin_func = self.process_dict[object_id].get_stderr
         else:
             raise AsyncProcessError(
                 "The stdout or stderr flag in id %s is missing" % str(id)
             )
         return stdin_func
 
+    def _create_param_stdin_process(self, param_stdin_funcs, param_val, param):
+        """Helper methods to create parameter stdin process.
+
+        Args:
+            module_descr (dict): The module description
+            param_stdin_funcs(dict): The dictionary with the stdout/stderr
+                                     functions
+            param_val(str): The value of parameter of the module
+            param(str): The parameter name of the module
+
+        Returns:
+            stdin_func(Process): An object of type Process that
+                                 contains the module name, the
+                                 parameter list and stdin definitions
+            filter(str): A string to filter stdout e.g. "max" for r.univar
+        """
+        for mod in self.process_dict:
+            p_splitted = param_val.split(f"{mod}::")
+            p_len = len(p_splitted)
+            for i in range(1, p_len):
+                object_id = mod
+                method = get_param_stdin_part(p_splitted[i])
+                rest_str = p_splitted[i].replace(method, "")
+                filter = None
+                if rest_str.startswith("::"):
+                    filter = get_param_stdin_part(rest_str)
+
+                if "stdout" == method:
+                    stdin_func = self.process_dict[object_id].get_stdout
+                elif "stderr" == method:
+                    stdin_func = self.process_dict[object_id].get_stderr
+                else:
+                    raise AsyncProcessError(
+                        f"The stdout or stderr flag in id {id} is missing"
+                    )
+                func_str = f"{object_id}::{method}"
+                func_name = f"PARAM_STDIN_FUNC_{self.stdin_num}"
+                if filter:
+                    func_str += f"::{filter}"
+                    param += f"::{filter}"
+
+                param_stdin_funcs[self.stdin_num] = stdin_func
+                param_val = param_val.replace(func_str, func_name)
+                self.stdin_num += 1
+
+        return param_val
+
     def _create_exec_process(self, module_descr):
         """Analyse a grass process description dict and create a Process
         that is used to execute a common Linux binary.
@@ -765,10 +827,8 @@ class ProcessChainConverter(object):
         executable = module_descr["exe"]
 
         params = []
-
         if "params" in module_descr:
             for search_string in module_descr["params"]:
-
                 # Search for file identifiers and generate the temporary file
                 # path
                 if "$file" in search_string and "::" in search_string:
@@ -833,7 +893,6 @@ class ProcessChainConverter(object):
                 or entry["import_descr"]["type"].lower() == "vector"
                 or entry["import_descr"]["type"].lower() == "file"
             ):
-
                 rvf_downimport_commands = (
                     self._get_raster_vector_file_download_import_command(entry)
                 )
@@ -930,7 +989,6 @@ class ProcessChainConverter(object):
 
         if "inputs" in module_descr:
             for key in module_descr["inputs"]:
-
                 search_string = str(module_descr["inputs"][key])
 
                 # Search for file identifiers and generate the temporary file
@@ -949,7 +1007,6 @@ class ProcessChainConverter(object):
                     # Check for mapset in input name and append it
                     # to the list of required mapsets
                     if "@" in str(module_descr["inputs"][key]):
-
                         # Mapset names are after an @ symbol
                         # Mapsets in expressions can be detected by replacing
                         # the symbols like *, +, :, /, {, (,},], ... by spaces
@@ -997,7 +1054,6 @@ class ProcessChainConverter(object):
         if "outputs" in module_descr:
             for key in module_descr["outputs"]:
                 if "name" in module_descr["outputs"][key]:
-
                     search_string = module_descr["outputs"][key]["name"]
 
                     # Search for file identifiers and generate the temporary
@@ -1129,7 +1185,6 @@ class ProcessChainConverter(object):
         executable = module_descr["executable"]
         if "parameters" in module_descr:
             for search_string in module_descr["parameters"]:
-
                 # Search for file identifiers and generate the temporary file
                 # path
                 if "$file" in search_string and "::" in search_string:
@@ -1167,7 +1222,7 @@ class ProcessChainConverter(object):
         return p
 
     def _add_grass_module_input_parameter_to_list(
-        self, module_descr, params, id
+        self, module_descr, params, param_stdin_funcs, id
     ):
         """Helper method to set the input parameters of a grass module and add
         them to the params list.
@@ -1177,6 +1232,8 @@ class ProcessChainConverter(object):
             params (list): The list of the grass module inputs parameters with
                            param=value entries (here the input parameter are
                            added)
+            param_stdin_funcs (dict): The dictonary with the stdin parameter
+                                      functions
             id (str): The id of this process in the process chain
         """
         if isinstance(module_descr["inputs"], list) is False:
@@ -1211,6 +1268,14 @@ class ProcessChainConverter(object):
                         file_id
                     ] = self.generate_temp_file_path()
                 param = "%s=%s" % (param, self.temporary_pc_files[file_id])
+            elif "::" in value and value.split("::")[1] in [
+                "stdout",
+                "stderr",
+            ]:
+                param_val = self._create_param_stdin_process(
+                    param_stdin_funcs, value, param
+                )
+                param = f"{param}={param_val}"
             else:
                 param = "%s=%s" % (param, value)
                 # Check for mapset in input name and append it,
@@ -1256,7 +1321,7 @@ class ProcessChainConverter(object):
             params.append(param)
 
     def _add_grass_module_output_parameter_to_list(
-        self, module_descr, params, id, module_name
+        self, module_descr, params, param_stdin_funcs, id, module_name
     ):
         """Helper method to set the output parameters of a grass module and add
         them to the params list. If export is in the output parameter the
@@ -1267,6 +1332,8 @@ class ProcessChainConverter(object):
             params (list): The list of the grass module parameters with
                            param=value entries (here the output parameter are
                            added)
+            param_stdin_funcs (dict): The dictonary with the stdin parameter
+                                      functions
             id (str): The id of this process in the process chain
             module_name (str): The name of the grass module
 
@@ -1342,6 +1409,15 @@ class ProcessChainConverter(object):
                         file_id,
                         output["export"]["format"].lower(),
                     )
+            elif "::" in value and value.split("::")[1] in [
+                "stdout",
+                "stderr",
+            ]:
+                id = module_descr["id"]
+                param_val = self._create_param_stdin_process(
+                    param_stdin_funcs, value, param
+                )
+                param = f"{param}={param_val}"
             else:
                 param = "%s=%s" % (param, value)
             params.append(param)
