@@ -36,10 +36,10 @@ from .messages_logger import MessageLogger
 __license__ = "GPLv3"
 __author__ = "Sören Gebbert"
 __copyright__ = (
-    "Copyright 2016-2018, Sören Gebbert and mundialis GmbH & Co. KG"
+    "Copyright 2016-2023, Sören Gebbert, Anika Weinmann and mundialis GmbH & "
+    "Co. KG"
 )
-__maintainer__ = "Sören Gebbert"
-__email__ = "soerengebbert@googlemail.com"
+__maintainer__ = "mundialis"
 
 
 class GrassInitError(Exception):
@@ -87,7 +87,6 @@ class GrassEnvironment(ProcessLogging):
     """This class saves and sets grass environment variables"""
 
     def __init__(self):
-
         ProcessLogging.__init__(self)
         self.env = {
             "GISBASE": "",
@@ -113,8 +112,8 @@ class GrassEnvironment(ProcessLogging):
 
         """
         self.env["GIS_LOCK"] = str(os.getpid())
-        self.env["HOME"] = "/tmp/"
         self.env["GISBASE"] = grass_gis_base
+        self.env["HOME"] = os.getenv("HOME", "/tmp/")
         self.env["GRASS_MESSAGE_FORMAT"] = "plain"
         self.env["GRASS_SKIP_MAPSET_OWNER_CHECK"] = "1"
         self.env["GRASS_TGIS_RAISE_ON_ERROR"] = "1"
@@ -122,7 +121,7 @@ class GrassEnvironment(ProcessLogging):
         self.env["LD_LIBRARY_PATH"] = str(
             os.path.join(self.env["GISBASE"], "lib")
         )
-        self.env["GRASS_VERSION"] = "7.7.svn"
+        self.env["GRASS_VERSION"] = "8"
         self.env["GRASS_ADDON_PATH"] = grass_addon_path
         self.env["GRASS_ADDON_BASE"] = grass_addon_path
         if os.name != "posix":
@@ -166,18 +165,14 @@ class GrassEnvironment(ProcessLogging):
                 )
 
     def set(self):
-        # # for debugging in ephemeral_processing.py (s. also process_queue.py)
-        # for var in [
-        #         'GISRC', 'GISBASE', 'LD_LIBRARY_PATH',
-        #         'GRASS_ADDON_PATH', 'GIS_LOCK']:
-        #     if var in os.environ:
-        #         del os.environ[var]
         for key in self.env:
-            try:
-                value = self.env[key]
-                origValue = os.getenv(key)
+            value = self.env[key]
+            # use self.env and enviroment variable values
+            if key in ["PATH", "PYTHONPATH"]:
+                origValue = os.getenv(key, None)
                 if origValue:
                     value += ":" + origValue
+            try:
                 os.putenv(key, value)
                 os.environ[key] = value
                 self.log_debug(key + "=" + value)
@@ -301,7 +296,6 @@ class GrassWindFile(ProcessLogging):
 
 class GrassModuleRunner(ProcessLogging):
     def __init__(self, grassbase, grass_addon_path):
-
         ProcessLogging.__init__(self)
 
         self.grassbase = grassbase
