@@ -4,7 +4,7 @@
 # performance processing of geographical data that uses GRASS GIS for
 # computational tasks. For details, see https://actinia.mundialis.de/
 #
-# Copyright (c) 2016-2022 Sören Gebbert and mundialis GmbH & Co. KG
+# Copyright (c) 2016-2024 Sören Gebbert and mundialis GmbH & Co. KG
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -36,7 +36,7 @@ from actinia_core.core.stac_exporter_interface import STACExporter
 __license__ = "GPLv3"
 __author__ = "Sören Gebbert"
 __copyright__ = (
-    "Copyright 2016-2022, Sören Gebbert and mundialis GmbH & Co. KG"
+    "Copyright 2016-2024, Sören Gebbert and mundialis GmbH & Co. KG"
 )
 __maintainer__ = "mundialis"
 __email__ = "info@mundialis.de"
@@ -104,7 +104,6 @@ class EphemeralProcessingWithExport(EphemeralProcessing):
         file_name = raster_name.split("@")[0] + suffix
 
         if use_raster_region is True:
-
             p = Process(
                 exec_type="grass",
                 executable="g.region",
@@ -149,11 +148,6 @@ class EphemeralProcessingWithExport(EphemeralProcessing):
             create_opts += ",TILED=YES"
 
         args.append(create_opts)
-        # current workaround due to color table export
-        # COG bug in GDAL, see https://github.com/OSGeo/gdal/issues/2946
-        # TODO: DELETE AND TEST ONCE GDAL 3.1.4 HAS BEEN RELEASED
-        if format == "COG":
-            args.append("-c")
 
         if additional_options:
             args.extend(additional_options)
@@ -411,7 +405,6 @@ class EphemeralProcessingWithExport(EphemeralProcessing):
         """
 
         for resource in self.resource_export_list:
-
             # Check for termination requests between the exports
             if (
                 bool(
@@ -432,7 +425,6 @@ class EphemeralProcessingWithExport(EphemeralProcessing):
                 "file",
                 "strds",
             ]:
-
                 output_type = resource["export"]["type"]
                 output_path = None
 
