@@ -286,10 +286,7 @@ process_chain_5_stdout = {
 
 
 class JobResumptionProcessingTestCase(ActiniaResourceTestCaseBase):
-
-    cfg_file = os.environ.get(
-        "ACTINIA_CUSTOM_TEST_CFG", "/etc/default/actinia"
-    )
+    cfg_file = os.environ.get("DEFAULT_CONFIG_PATH", "/etc/default/actinia")
     tmp_cfg_file = "%s_tmp" % cfg_file
     save_interim_results_value = None
     endpoint = "/locations/nc_spm_08/processing_async"
@@ -919,7 +916,6 @@ class JobResumptionProcessingTestCase(ActiniaResourceTestCaseBase):
 
 
 class JobResumptionProcessingExportTestCase(JobResumptionProcessingTestCase):
-
     endpoint = "/locations/nc_spm_08/processing_async_export"
     resource_user_id = None
     resource_resource_id = None
@@ -1079,7 +1075,6 @@ class JobResumptionProcessingExportTestCase(JobResumptionProcessingTestCase):
 class JobResumptionPersistentProcessingTestCase(
     JobResumptionProcessingTestCase
 ):
-
     location = "nc_spm_08"
     mapset = "test_mapset"
     endpoint = "/locations/%s/mapsets/%s/processing_async" % (location, mapset)
@@ -1154,7 +1149,6 @@ class JobResumptionPersistentProcessingTestCase(
 
 
 class JobResumptionErrorTestCase(ActiniaResourceTestCaseBase):
-
     endpoint = "/locations/nc_spm_08/processing_async"
 
     def test_job_resumption_config_error(self):
